@@ -1,23 +1,11 @@
-"""
-.. module:: CMalConvAttackEvasion
-	:synopsis: Class performs evasion attacks against end to end classifiers, under different constraints.
-
-.. moduleauthor:: Luca Demetrio <luca.demetrio@dibris.unige.it>
-
-"""
 import copy
 from abc import abstractmethod
 
 import numpy as np
 import torch
-
 from secml.adv.attacks import CAttackEvasion
 from secml.array import CArray
 from secml.array.c_dense import CDense
-from secml.data import CDataset
-from secml.optim.constraints import CConstraint
-from secml.optim.function import CFunction
-from secml.optim.optimizers import COptimizer
 from secml.settings import SECML_PYTORCH_USE_CUDA
 
 from secml_malware.models import CClassifierEnd2EndMalware
@@ -26,8 +14,10 @@ use_cuda = torch.cuda.is_available() and SECML_PYTORCH_USE_CUDA
 
 
 class CEnd2EndMalwareEvasion(CAttackEvasion):
-	"""Base abstract class for implementing end-to-end evasion attacks against malware detectors.
 	"""
+	Base abstract class for implementing end-to-end evasion attacks against malware detectors.
+	"""
+
 	def _objective_function_gradient(self, x):
 		pass
 
@@ -35,15 +25,15 @@ class CEnd2EndMalwareEvasion(CAttackEvasion):
 		pass
 
 	def __init__(
-		self,
-		end2end_model: CClassifierEnd2EndMalware,
-		indexes_to_perturb: list,
-		iterations: int = 100,
-		is_debug: bool = False,
-		random_init: bool = False,
-		threshold : float = 0.5,
-		penalty_regularizer : float = 0,
-		store_checkpoints : int = None
+			self,
+			end2end_model: CClassifierEnd2EndMalware,
+			indexes_to_perturb: list,
+			iterations: int = 100,
+			is_debug: bool = False,
+			random_init: bool = False,
+			threshold: float = 0.5,
+			penalty_regularizer: float = 0,
+			store_checkpoints: int = None
 
 	):
 		CAttackEvasion.__init__(
@@ -84,14 +74,14 @@ class CEnd2EndMalwareEvasion(CAttackEvasion):
 	def objective_function(self, x):
 		pass
 
-
 	@abstractmethod
-	def loss_function_gradient(self, original_x : CArray, adv_x : CArray, penalty_term : torch.Tensor):
+	def loss_function_gradient(self, original_x: CArray, adv_x: CArray, penalty_term: torch.Tensor):
 		raise NotImplementedError("This is only an abstract method")
 
 	def _run(self, x0, y0, x_init=None):
 		"""
 		Tries to achieve evasion against and end-to-end classifier.
+
 		Parameters
 		----------
 		x0 : CArray
@@ -124,9 +114,9 @@ class CEnd2EndMalwareEvasion(CAttackEvasion):
 		if use_cuda:
 			self._invalid_value = self._invalid_value.cuda()
 
-		E = self.get_embedded_byte_matrix()
+		E = self._get_embedded_byte_matrix()
 		if self.random_init:
-			x_init = self.randomize_values_for_attack(x_init)
+			x_init = self._randomize_values_for_attack(x_init)
 
 		if self.is_debug:
 			print("> Beginning new sample evasion...")
@@ -135,7 +125,8 @@ class CEnd2EndMalwareEvasion(CAttackEvasion):
 		x_init = self.apply_feature_mapping(x_init)
 		for t in range(self.iterations):
 			if current_conf < self.threshold:
-				print(f"Stopped at confidence below threshold: {current_conf}/{self.threshold}")
+				if self.is_debug:
+					print(f"Stopped at confidence below threshold: {current_conf}/{self.threshold}")
 				break
 
 			penalty_term = self.compute_penalty_term(x0, x_init, self.penalty_regularizer)
@@ -162,25 +153,25 @@ class CEnd2EndMalwareEvasion(CAttackEvasion):
 			print(f'>AFTER INVERSION, CONFIDENCE SCORE: {current_conf}')
 		return x_init, current_conf
 
-	def get_embedded_byte_matrix(self):
+	def _get_embedded_byte_matrix(self):
 		return self.classifier.embed(
 			np.array([[i if i < 256 else 256 for i in range(self.max_input_length)]]),
 			transpose=False,
 		)[0][:257]
 
 	@abstractmethod
-	def apply_feature_mapping(self, x):
+	def apply_feature_mapping(self, x) -> CArray:
 		raise NotImplementedError("This method is abstract, you should implement it somewhere else!")
 
 	@abstractmethod
-	def invert_feature_mapping(self, x, x_adv):
+	def invert_feature_mapping(self, x, x_adv) -> CArray:
 		raise NotImplementedError("This method is abstract, you should implement it somewhere else!")
 
 	@abstractmethod
-	def infer_step(self, x_init):
+	def infer_step(self, x_init) -> CArray:
 		raise NotImplementedError("This method is abstract, you should implement it somewhere else!")
 
-	def randomize_values_for_attack(self, x_init):
+	def _randomize_values_for_attack(self, x_init):
 		min_val = 0 + self.shift_values
 		max_val = 255 + self.shift_values
 		x_init[self.indexes_to_perturb] = CDense(
@@ -191,18 +182,35 @@ class CEnd2EndMalwareEvasion(CAttackEvasion):
 		return x_init
 
 	@abstractmethod
-	def compute_penalty_term(self, original_x: CArray, adv_x: CArray, par: float):
+	def compute_penalty_term(self, original_x: CArray, adv_x: CArray, par: float) -> float:
 		raise NotImplementedError("This method is abstract, you should implement it somewhere else!")
 
 	@abstractmethod
-	def optimization_solver(self, E, gradient_f, index_to_consider, x_init):
+	def optimization_solver(self, E, gradient_f, index_to_consider, x_init) -> CArray:
 		raise NotImplementedError("This method is abstract, you should implement it somewhere else!")
 
+	def create_real_sample_from_adv(self, original_file_path: str, x_adv: CArray,
+									new_file_path: str = None) -> bytearray:
+		"""
+		Create a real adversarial example
 
-	def create_real_sample_from_adv(self, original_file_path : str, x_adv : CArray, new_file_path : str = None):
+		Parameters
+		----------
+		original_file_path : str
+			the original malware sample
+		x_adv : CArray
+			the perturbed malware sample, as created by the optimizer
+		new_file_path : str, optional, default None
+			the path where to save the adversarial malware. Leave None to not save the result to disk
+
+		Returns
+		-------
+		bytearray
+			the adversarial malware, as string of bytes
+		"""
 		with open(original_file_path, 'rb') as f:
 			code = bytearray(f.read())
-		padding_index = x_adv.find(x_adv ==  self.classifier.get_embedding_value())
+		padding_index = x_adv.find(x_adv == self.classifier.get_embedding_value())
 		padded_x_adv = copy.copy(x_adv)
 		if padding_index:
 			padded_x_adv = padded_x_adv[0, :padding_index[0]]
@@ -214,4 +222,4 @@ class CEnd2EndMalwareEvasion(CAttackEvasion):
 		if new_file_path:
 			with open(new_file_path, 'wb') as f:
 				f.write(code)
-		return code
\ No newline at end of file
+		return code
