@@ -19,6 +19,30 @@ class CFormatExploitEvasion(CDiscreteBytesEvasion):
 			penalty_regularizer: float = 0,
 			chunk_hyper_parameter : int = None,
 	):
+		"""
+		Creates the evasion object
+
+		Parameters
+		----------
+		end2end_model : CClassifierEnd2EndMalware
+			the target end-to-end model
+		preferable_extension_amount : int, optional, default 512
+			the number of bytes to inject before the first section, modulo file alignment. Default 512.
+		pe_header_extension : int, optional, default 512
+			the number of bytes to inject as new DOS header, modulo file alignment. Default 512.
+		iterations : int, optional, default 100
+			the number of iterations of the optimizer
+		is_debug : bool, optional, default False
+			if True, prints debug information during the optimization
+		random_init : bool, optional, default False
+			if True, it randomizes the locations set by index_to_perturb before starting the optimization
+		threshold : float, optional, default 0
+			the detection threshold to bypass. Default is 0
+		penalty_regularizer : float
+			the reularization parameter, Default is 0
+		chunk_hyper_parameter : int, optional, default 256
+			how many bytes to optimize at each round. Default is 256
+		"""
 		super(CFormatExploitEvasion, self).__init__(
 			end2end_model=end2end_model,
 			index_to_perturb=[],
@@ -58,7 +82,24 @@ class CFormatExploitEvasion(CDiscreteBytesEvasion):
 														 index_to_perturb_sections]
 		return x_init, indexes_to_perturb
 
-	def create_real_sample_from_adv(self, original_file_path : str, x_adv : CArray, new_file_path : str = None):
+	def create_real_sample_from_adv(self, original_file_path : str, x_adv : CArray, new_file_path : str = None) -> bytearray:
+		"""
+		Create a real adversarial example
+
+		Parameters
+		----------
+		original_file_path : str
+			the original malware sample
+		x_adv : CArray
+			the perturbed malware sample, as created by the optimizer
+		new_file_path : str, optional, default None
+			the path where to save the adversarial malware. Leave None to not save the result to disk
+
+		Returns
+		-------
+		bytearray
+			the adversarial malware, as string of bytes
+		"""
 		with open(original_file_path, 'rb') as f:
 			code = bytearray(f.read())
 		original_x = CArray([numpy.frombuffer(code, dtype=numpy.uint8).astype(numpy.uint16)])
