@@ -36,8 +36,14 @@ class CClassifierEmber(CClassifier):
 			EMBER features
 		"""
 		extractor = PEFeatureExtractor(2, print_feature_warning=False)
-		x_bytes = bytes(x.astype(np.int).tolist()[0])
-		features = CArray([np.array(extractor.feature_vector(x_bytes), dtype=np.float32)])
+		x = x.atleast_2d()
+		size = x.shape[0]
+		features = []
+		for i in range(size):
+			x_i = x[i, :]
+			x_bytes = bytes(x_i.astype(np.int).tolist()[0])
+			features.append(np.array(extractor.feature_vector(x_bytes), dtype=np.float32))
+		features = CArray(features)
 		return features
 
 	def _backward(self, w):
@@ -53,6 +59,8 @@ class CClassifierEmber(CClassifier):
 		return booster
 
 	def _forward(self, x):
-		confidence = self._lightgbm_model.predict(x.tondarray())[0]
-		confidence = CArray([1 - confidence, confidence])
+		x = x.atleast_2d()
+		scores = self._lightgbm_model.predict(x.tondarray())
+		confidence = [[1 - c, c] for c in scores]
+		confidence = CArray(confidence)
 		return confidence
