@@ -1,12 +1,3 @@
-"""
-.. module:: Exceptions
-	:synopsis: Useful and meaningful exception for module.
-
-.. moduleauthor:: Luca Demetrio <luca.demetrio@dibris.unige.it>
-
-"""
-
-
 class FileNotExistsException(Exception):
 	def __init__(self, message):
 		super(FileNotExistsException, self).__init__(message)
